"""
Simple example that counts the number of changes in an osm diff file.

Shows how to detect the different kind of modifications and how to
use the handler generator function instead of a handler class.
"""
import osmium as o
import sys

class Stats(object):

    def __init__(self):
        self.added = 0
        self.modified = 0
        self.deleted = 0

    def __call__(self, o):
        if o.deleted:
            self.deleted += 1
        elif o.version == 1:
            self.added += 1
        else:
            self.modified += 1

    def outstats(self, prefix):
        print("%s added: %d" % (prefix, self.added))
        print("%s modified: %d" % (prefix, self.modified))
        print("%s deleted: %d" % (prefix, self.deleted))


def main(osmfile):
    nodes = Stats()
    ways = Stats()
    rels = Stats()

    h = o.make_simple_handler(node=nodes, way=ways, relation=rels)

    h.apply_file(osmfile)

    nodes.outstats("Nodes")
    ways.outstats("Ways")
    rels.outstats("Relations")

    return 0

if __name__ == '__main__':
    if len(sys.argv) != 2:
        print("Usage: python %s <osmfile>" % sys.argv[0])
        sys.exit(-1)

    exit(main(sys.argv[1]))



""" Tests for the pyosmium-get-changes script.
"""

from helpers import load_script
from nose.tools import *
import unittest
from io import BytesIO
from os import path as osp
from textwrap import dedent
import sys
import tempfile
from os import unlink

try:
    from cStringIO import StringIO
except:
    from io import StringIO

from requests import Session

try:
    import http.cookiejar as cookiejarlib
except ImportError:
    import cookielib as cookiejarlib

try:
    from urllib.error import URLError
except ImportError:
    from urllib2 import URLError

try:
    from unittest.mock import MagicMock, DEFAULT
except ImportError:
    from mock import MagicMock, DEFAULT

class Capturing(list):
    def __enter__(self):
        self._stdout = sys.stdout
        sys.stdout = self._stringio = StringIO()
        return self
    def __exit__(self, *args):
        self.extend(self._stringio.getvalue().splitlines())
        del self._stringio    # free up some memory
        sys.stdout = self._stdout

class RequestsResponses(BytesIO):

    def __init__(self, bytes):
       super(RequestsResponses, self).__init__(bytes)
       self.content = bytes

    def iter_lines(self):
       return self.readlines()

class TestPyosmiumGetChanges(unittest.TestCase):

    def setUp(self):
        self.script = load_script(osp.join(osp.realpath(__file__),
                                           "../../tools/pyosmium-get-changes"))
        self.url_mock = MagicMock()
        self.urls = dict()
        self.url_mock.side_effect = lambda url,data=None,timeout=None : self.urls[url.get_full_url()]
        self.script['rserv'].urlrequest.OpenerDirector.open = self.url_mock

        self.urlreq_mock = MagicMock()
        self.urlreq_mock.side_effect = lambda url,**kw : self.urls[url]
        self.script['rserv'].requests.Session.get = self.urlreq_mock

    def url(self, url, result):
        self.urls[url] = RequestsResponses(dedent(result).encode())

    def main(self, *args):
        with Capturing() as output:
            ret = self.script['main'](args)
            self.stdout = output
        return ret

    def test_init_id(self):
        assert_equals(0, self.main('-I', '453'))
        assert_equals(1, len(self.stdout))
        assert_equals('454', self.stdout[0])

    def test_init_date(self):
        self.url('https://planet.osm.org/replication/minute//state.txt',
                 """\
                    sequenceNumber=100
                    timestamp=2017-08-26T11\:04\:02Z
                 """)
        self.url('https://planet.osm.org/replication/minute//000/000/000.state.txt',
                 """\
                    sequenceNumber=0
                    timestamp=2016-08-26T11\:04\:02Z
                 """)
        assert_equals(0, self.main('-D', '2015-12-24T08:08:08Z'))
        assert_equals(1, len(self.stdout))
        assert_equals('1', self.stdout[0])

    def test_init_to_file(self):
        fd = tempfile.NamedTemporaryFile(dir=tempfile.gettempdir(), suffix='.seq', delete=False)
        fname = fd.name
        fd.close()

        assert_equals(0, self.main('-I', '453', '-f', fd.name))
        fd = open(fname, 'r')
        content = fd.read()
        try:
            assert_equals('454', content)
        finally:
            fd.close()
            unlink(fname)

    def test_init_from_seq_file(self):
        with tempfile.NamedTemporaryFile(dir=tempfile.gettempdir(), suffix='.seq', delete=False) as fd:
            fd.write('453'.encode('utf-8'))
            fname = fd.name

        assert_equals(0, self.main('-f', fname))
        fd = open(fname, 'r')
        content = fd.read()
        try:
            assert_equals('454', content)
        finally:
            fd.close()
            unlink(fname)

    def test_init_date_with_cookie(self):
        self.url('https://planet.osm.org/replication/minute//state.txt',
                 """\
                    sequenceNumber=100
                    timestamp=2017-08-26T11\:04\:02Z
                 """)
        self.url('https://planet.osm.org/replication/minute//000/000/000.state.txt',
                 """\
                    sequenceNumber=0
                    timestamp=2016-08-26T11\:04\:02Z
                 """)

        with tempfile.NamedTemporaryFile(dir=tempfile.gettempdir(), suffix='.cookie', delete=False) as fd:
            fname = fd.name
        cookie_jar = cookiejarlib.MozillaCookieJar(fname)
        cookie_jar.save()

        try:
            assert_equals(0, self.main('--cookie', fname, '-D', '2015-12-24T08:08:08Z'))
            assert_equals(1, len(self.stdout))
            assert_equals('1', self.stdout[0])
        finally:
            unlink(fname)



import osmium.osm.mutable
from ._osm import *

def create_mutable_node(node, **args):
    """ Create a mutable node replacing the properties given in the
        named parameters. Note that this function only creates a shallow
        copy which is still bound to the scope of the original object.
    """
    return osmium.osm.mutable.Node(base=node, **args)

def create_mutable_way(way, **args):
    """ Create a mutable way replacing the properties given in the
        named parameters. Note that this function only creates a shallow
        copy which is still bound to the scope of the original object.
    """
    return osmium.osm.mutable.Way(base=way, **args)

def create_mutable_relation(rel, **args):
    """ Create a mutable relation replacing the properties given in the
        named parameters. Note that this function only creates a shallow
        copy which is still bound to the scope of the original object.
    """
    return osmium.osm.mutable.Relation(base=rel, **args)

Node.replace = create_mutable_node
Way.replace = create_mutable_way
Relation.replace = create_mutable_relation

def _make_repr(attr_list):
    fmt_string = 'osmium.osm.{0}('\
                 + ', '.join(['{0}={{1.{0}!r}}'.format(x) for x in attr_list])\
                 + ')'

    return lambda o: fmt_string.format(o.__class__.__name__, o)

def _list_repr(obj):
    return 'osmium.osm.{}([{}])'.format(obj.__class__.__name__,
                                        ', '.join(map(repr, obj)))

def _list_elipse(obj):
    objects = ','.join(map(str, obj))
    if len(objects) > 50:
        objects = objects[:47] + '...'
    return objects

Location.__repr__ = lambda l: 'osmium.osm.Location(x={0.x!r}, y={0.y!r})'.format(l) \
                               if l.valid() else 'osmium.osm.Location()'
Location.__str__ = lambda l: '{:f}/{:f}'.format(l.lon_without_check(),
                                                l.lat_without_check()) \
                             if l.valid() else 'invalid'

Box.__repr__ = _make_repr(['bottom_left', 'top_right'])
Box.__str__ = lambda b: '({0.bottom_left!s} {0.top_right!s})'.format(b)

Tag.__repr__ = _make_repr(['k', 'v'])
Tag.__str__ = lambda t: '{0.k}={0.v}'.format(t)

TagList.__repr__ = lambda t: "osmium.osm.TagList({%s})" \
                              % ', '.join(["%r: %r" % (i.k, i.v) for i in t])
TagList.__str__ = lambda t: '{' + _list_elipse(t) + '}'

NodeRef.__repr__ = _make_repr(['ref', 'location'])
NodeRef.__str__ = lambda n: '{0.ref:d}@{0.location!s}'.format(n) \
                            if n.location.valid() else str(n.ref)

NodeRefList.__repr__ = _list_repr
NodeRefList.__str__ = lambda o: '[' + _list_elipse(o) + ']'

RelationMember.__repr__ = _make_repr(['ref', 'type', 'role'])
RelationMember.__str__ = lambda r: ('{0.type}{0.ref:d}@{0.role}' \
                                   if r.role else '{0.type}{0.ref:d}').format(r)

RelationMemberList.__repr__ = _list_repr
RelationMemberList.__str__ = lambda o: '[' + _list_elipse(o) + ']'

OSMObject.__repr__ = _make_repr(['id', 'deleted', 'visible', 'version', 'changeset',
                                 'uid', 'timestamp', 'user', 'tags'])

Node.__repr__ = _make_repr(['id', 'deleted', 'visible', 'version', 'changeset',
                            'uid', 'timestamp', 'user', 'tags', 'location'])
Node.__str__ = lambda n: 'n{0.id:d}: location={0.location!s} tags={0.tags!s}'\
                         .format(n)

Way.__repr__ = _make_repr(['id', 'deleted', 'visible', 'version', 'changeset',
                           'uid', 'timestamp', 'user', 'tags', 'nodes'])
Way.__str__ = lambda o: 'w{0.id:d}: nodes={0.nodes!s} tags={0.tags!s}' \
                         .format(o)

Relation.__repr__ = _make_repr(['id', 'deleted', 'visible', 'version', 'changeset',
                                'uid', 'timestamp', 'user', 'tags', 'members'])
Relation.__str__ = lambda o: 'r{0.id:d}: members={0.members!s}, tags={0.tags!s}' \
                             .format(o)

Changeset.__repr__ = _make_repr(['id', 'uid', 'created_at', 'closed_at', 'open',
                                 'num_changes', 'bounds', 'user', 'tags'])
Changeset.__str__ = lambda o: 'c{0.id:d}: closed_at={0.closed_at!s}, bounds={0.bounds!s}, tags={0.tags!s}' \
                              .format(o)



"""
Tests for all examples.
"""
from nose.tools import *
import unittest
import sys
from helpers import load_script
from os import path as osp

TEST_FILE='example-test.pbf'
TEST_DIFF='example-test.osc'

try:
    from StringIO import StringIO
except:
    from io import StringIO

class Capturing(list):
    def __enter__(self):
        self._stdout = sys.stdout
        sys.stdout = self._stringio = StringIO()
        return self
    def __exit__(self, *args):
        self.extend(self._stringio.getvalue().splitlines())
        del self._stringio    # free up some memory
        sys.stdout = self._stdout

def load_example(name):
    return load_script(osp.join(osp.realpath(__file__),
                                "..", "..", "examples", name + ".py"))

def test_amenity_list():
    script = load_example("amenity_list")

    with Capturing() as output:
        eq_(0, script['main'](TEST_FILE))

    eq_(835, len(output))
    eq_('1.535245 42.556681 parking', output[0].strip())
    eq_('1.570729 42.529562 parking         Aparcament Comunal', output[-1].strip())

def test_road_length():
    script = load_example("road_length")

    with Capturing() as output:
        eq_(0, script['main'](TEST_FILE))

    eq_(output, ["Total way length: 1590.82 km"])


def test_pub_names():
    script = load_example("pub_names")

    with Capturing() as output:
        eq_(0, script['main'](TEST_FILE))

    eq_(output, ['Kyu', 'Havana Club', "Mulligan's", 'Bar Broques',
                 'The Camden - English Pub', 'Aspen', 'el Raval'])

def test_osm_diff_stats():
    script = load_example("osm_diff_stats")

    with Capturing() as output:
        eq_(0, script['main'](TEST_DIFF))

    eq_(9, len(output))
    eq_('Nodes added: 305', output[0])
    eq_('Nodes modified: 192', output[1])
    eq_('Nodes deleted: 20', output[2])
    eq_('Ways added: 31', output[3])
    eq_('Ways modified: 93', output[4])
    eq_('Ways deleted: 0', output[5])
    eq_('Relations added: 0', output[6])
    eq_('Relations modified: 0', output[7])
    eq_('Relations deleted: 0', output[8])

def test_osm_file_stats():
    script = load_example("osm_file_stats")

    with Capturing() as output:
        eq_(0, script['main'](TEST_FILE))

    eq_(3, len(output))
    eq_('Nodes: 211100', output[0])
    eq_('Ways: 10315', output[1])
    eq_('Relations: 244', output[2])



""" Helper functions to communicate with replication servers.
"""

import requests
import urllib.request as urlrequest
import datetime as dt
from collections import namedtuple
from math import ceil
from osmium import MergeInputReader
from osmium import io as oio
from osmium import version

import logging

LOG = logging.getLogger('pyosmium')
LOG.addHandler(logging.NullHandler())

OsmosisState = namedtuple('OsmosisState', ['sequence', 'timestamp'])
DownloadResult = namedtuple('DownloadResult', ['id', 'reader', 'newest'])

class ReplicationServer:
    """ Represents a server that publishes replication data. Replication
        change files allow to keep local OSM data up-to-date without downloading
        the full dataset again.
    """

    def __init__(self, url, diff_type='osc.gz'):
        self.baseurl = url
        self.diff_type = diff_type
        self.session = requests.Session()

    def make_request(self, url):
        headers = {"User-Agent" : "pyosmium/{}".format(version.pyosmium_release)}
        return urlrequest.Request(url, headers=headers)

    def open_url(self, url):
        """ Download a resource from the given URL and return a byte sequence
            of the content.

            This method has no support for cookies or any special authentication
            methods. If you need these, you have to provide your own custom URL
            opener. The method has to return an object which supports the
            `read()` and `readline()` methods to access the content. Example::

                def my_open_url(self, url):
                    opener = urlrequest.build_opener()
                    opener.addheaders = [('X-Fancy-Header', 'important_content')]
                    return opener.open(url)

                svr = ReplicationServer()
                svr.open_url = my_open_url
        """
        headers = dict()
        for h in url.header_items():
            headers[h[0]] = h[1]
        return self.session.get(url.get_full_url(), headers=headers, stream=True)

    def collect_diffs(self, start_id, max_size=1024):
        """ Create a MergeInputReader and download diffs starting with sequence
            id `start_id` into it. `max_size`
            restricts the number of diffs that are downloaded. The download
            stops as soon as either a diff cannot be downloaded or the
            unpacked data in memory exceeds `max_size` kB.

            If some data was downloaded, returns a namedtuple with three fields:
            `id` contains the sequence id of the last downloaded diff, `reader`
            contains the MergeInputReader with the data and `newest` is a
            sequence id of the most recent diff available.

            Returns None if there was an error during download or no new
            data was available.
        """
        left_size = max_size * 1024
        current_id = start_id

        # must not read data newer than the published sequence id
        # or we might end up reading partial data
        newest = self.get_state_info()

        if newest is None or current_id > newest.sequence:
            return None

        rd = MergeInputReader()

        while left_size > 0 and current_id <= newest.sequence:
            try:
                diffdata = self.get_diff_block(current_id)
            except:
                LOG.debug("Error during diff download. Bailing out.")
                diffdata = ''
            if len(diffdata) == 0:
                if start_id == current_id:
                    return None
                break

            left_size -= rd.add_buffer(diffdata, self.diff_type)
            LOG.debug("Downloaded change %d. (%d kB available in download buffer)",
                      current_id, left_size / 1024)
            current_id += 1

        return DownloadResult(current_id - 1, rd, newest.sequence)

    def apply_diffs(self, handler, start_id, max_size=1024, idx="", simplify=True):
        """ Download diffs starting with sequence id `start_id`, merge them
            together and then apply them to handler `handler`. `max_size`
            restricts the number of diffs that are downloaded. The download
            stops as soon as either a diff cannot be downloaded or the
            unpacked data in memory exceeds `max_size` kB.

            If `idx` is set, a location cache will be created and applied to
            the way nodes. You should be aware that diff files usually do not
            contain the complete set of nodes when a way is modified. That means
            that you cannot just create a new location cache, apply it to a diff
            and expect to get complete way geometries back. Instead you need to
            do an initial data import using a persistent location cache to
            obtain a full set of node locations and then reuse this location
            cache here when applying diffs.

            Diffs may contain multiple versions of the same object when it was
            changed multiple times during the period covered by the diff. If
            `simplify` is set to False then all versions are returned. If it
            is True (the default) then only the most recent version will be
            sent to the handler.

            The function returns the sequence id of the last diff that was
            downloaded or None if the download failed completely.
        """
        diffs = self.collect_diffs(start_id, max_size)

        if diffs is None:
            return None

        diffs.reader.apply(handler, idx=idx, simplify=simplify)

        return diffs.id

    def apply_diffs_to_file(self, infile, outfile, start_id, max_size=1024,
                            set_replication_header=True, extra_headers=None,
                            outformat=None):
        """ Download diffs starting with sequence id `start_id`, merge them
            with the data from the OSM file named `infile` and write the result
            into a file with the name `outfile`. The output file must not yet
            exist.

            `max_size` restricts the number of diffs that are downloaded. The
            download stops as soon as either a diff cannot be downloaded or the
            unpacked data in memory exceeds `max_size` kB.

            If `set_replication_header` is true then the URL of the replication
            server and the sequence id and timestamp of the last diff applied
            will be written into the `writer`. Note that this currently works
            only for the PBF format.

            `extra_headers` is a dict with additional header fields to be set.
            Most notably, the 'generator' can be set this way.

            `outformat` sets the format of the output file. If None, the format
            is determined from the file name.

            The function returns a tuple of last downloaded sequence id and
            newest available sequence id if new data has been written or None
            if no data was available or the download failed completely.
        """
        diffs = self.collect_diffs(start_id, max_size)

        if diffs is None:
            return None

        reader = oio.Reader(infile)
        has_history = reader.header().has_multiple_object_versions

        h = oio.Header()
        h.has_multiple_object_versions = has_history
        if set_replication_header:
            h.set("osmosis_replication_base_url", self.baseurl)
            h.set("osmosis_replication_sequence_number", str(diffs.id))
            info = self.get_state_info(diffs.id)
            if info is not None:
                h.set("osmosis_replication_timestamp", info.timestamp.strftime("%Y-%m-%dT%H:%M:%SZ"))
        if extra_headers is not None:
            for k, v in extra_headers.items():
                h.set(k, v)

        if outformat is None:
            of = oio.File(outfile)
        else:
            of = oio.File(outfile, outformat)

        of.has_multiple_object_versions = has_history
        writer = oio.Writer(of, h)

        LOG.debug("Merging changes into OSM file.")

        diffs.reader.apply_to_reader(reader, writer, has_history)

        reader.close()
        writer.close()

        return (diffs.id, diffs.newest)


    def timestamp_to_sequence(self, timestamp, balanced_search=False):
        """ Get the sequence number of the replication file that contains the
            given timestamp. The search algorithm is optimised for replication
            servers that publish updates in regular intervals. For servers
            with irregular change file publication dates 'balanced_search`
            should be set to true so that a standard binary search for the
            sequence will be used. The default is good for all known
            OSM replication services.
        """

        # get the current timestamp from the server
        upper = self.get_state_info()

        if upper is None:
            return None
        if timestamp >= upper.timestamp or upper.sequence <= 0:
            return upper.sequence

        # find a state file that is before the required timestamp
        lower = None
        lowerid = 0
        while lower is None:
            LOG.debug("Trying with Id %s", lowerid)
            lower = self.get_state_info(lowerid)

            if lower is not None and lower.timestamp >= timestamp:
                if lower.sequence == 0 or lower.sequence + 1 >= upper.sequence:
                    return lower.sequence
                upper = lower
                lower = None
                lowerid = 0

            if lower is None:
                # no lower yet, so try a higher id (binary search wise)
                newid = int((lowerid + upper.sequence) / 2)
                if newid <= lowerid:
                    # nothing suitable found, so upper is probably the best we can do
                    return upper.sequence
                lowerid = newid

        # Now do a binary search between upper and lower.
        # We could be clever here and compute the most likely state file
        # by interpolating over the timestamps but that creates a whole ton of
        # special cases that need to be handled correctly.
        while True:
            if balanced_search:
                base_splitid = int((lower.sequence + upper.sequence) / 2)
            else:
                ts_int = (upper.timestamp - lower.timestamp).total_seconds()
                seq_int = upper.sequence - lower.sequence
                goal = (timestamp - lower.timestamp).total_seconds()
                base_splitid = lower.sequence + ceil(goal * seq_int / ts_int)
                if base_splitid >= upper.sequence:
                    base_splitid = upper.sequence - 1
            split = self.get_state_info(base_splitid)

            if split is None:
                # file missing, search the next towards lower
                splitid = base_splitid - 1
                while split is None and splitid > lower.sequence:
                    split = self.get_state_info(splitid)
                    splitid -= 1
            if split is None:
                # still nothing? search towards upper
                splitid = base_splitid + 1
                while split is None and splitid < upper.sequence:
                    split = self.get_state_info(splitid)
                    splitid += 1
            if split is None:
                # still nothing? Then lower has to do
                return lower.sequence

            # set new boundary
            if split.timestamp < timestamp:
                lower = split
            else:
                upper = split

            if lower.sequence + 1 >= upper.sequence:
                return lower.sequence


    def get_state_info(self, seq=None, retries=2):
        """ Downloads and returns the state information for the given
            sequence. If the download is successful, a namedtuple with
            `sequence` and `timestamp` is returned, otherwise the function
            returns `None`. `retries` sets the number of times the download
            is retried when pyosmium detects a truncated state file.
        """
        for _ in range(retries + 1):
            try:
                response = self.open_url(self.make_request(self.get_state_url(seq)))
            except Exception as err:
                LOG.debug("Loading state info %s failed with: %s", seq, str(err))
                return None

            ts = None
            seq = None
            if hasattr(response, 'iter_lines'):
                # generated by requests
                get_line = response.iter_lines
            else:
                # generated by urllib.request
                get_line = response.readline

            for line in get_line():
                line = line.decode('utf-8')
                if '#' in line:
                    line = line[0:line.index('#')]
                else:
                    line = line.strip()
                if line:
                    kv = line.split('=', 2)
                    if len(kv) != 2:
                        return None
                    if kv[0] == 'sequenceNumber':
                        seq = int(kv[1])
                    elif kv[0] == 'timestamp':
                        try:
                            ts = dt.datetime.strptime(kv[1], "%Y-%m-%dT%H\\:%M\\:%SZ")
                        except ValueError:
                            break
                        ts = ts.replace(tzinfo=dt.timezone.utc)

            if ts is not None and seq is not None:
                return OsmosisState(sequence=seq, timestamp=ts)

        return None

    def get_diff_block(self, seq):
        """ Downloads the diff with the given sequence number and returns
            it as a byte sequence. Throws a :code:`urllib.error.HTTPError`
            (or :code:`urllib2.HTTPError` in python2)
            if the file cannot be downloaded.
        """
        resp = self.open_url(self.make_request(self.get_diff_url(seq)))
        if hasattr(resp, 'content'):
            # generated by requests
            return resp.content
        else:
            # generated by urllib.request
            return resp.read()


    def get_state_url(self, seq):
        """ Returns the URL of the state.txt files for a given sequence id.

            If seq is `None` the URL for the latest state info is returned,
            i.e. the state file in the root directory of the replication
            service.
        """
        if seq is None:
            return self.baseurl + '/state.txt'

        return '%s/%03i/%03i/%03i.state.txt' % \
               (self.baseurl, seq / 1000000, (seq % 1000000) / 1000, seq % 1000)


    def get_diff_url(self, seq):
        """ Returns the URL to the diff file for the given sequence id.
        """
        return '%s/%03i/%03i/%03i.%s' % \
               (self.baseurl,
                seq / 1000000, (seq % 1000000) / 1000, seq % 1000,
                self.diff_type)



from osmium._osmium import SimpleHandler

def make_simple_handler(node=None, way=None, relation=None, area=None):
    """ Convenience function that creates a `SimpleHandler` from a set of
        callback functions. Each of the parameters takes an optional callable
        that must expect a single positional parameter with the object being
        processed.
    """
    class __HandlerWithCallbacks(SimpleHandler):
        pass

    if node is not None:
        __HandlerWithCallbacks.node = staticmethod(node)
    if way is not None:
        __HandlerWithCallbacks.way = staticmethod(way)
    if relation is not None:
        __HandlerWithCallbacks.relation = staticmethod(relation)
    if area is not None:
        __HandlerWithCallbacks.area = staticmethod(area)

    return __HandlerWithCallbacks()



"""
Extract all objects with an amenity tag from an osm file and list them
with their name and position.

This example shows how geometries from osmium objects can be imported
into shapely using the WKBFactory.
"""
import osmium as o
import sys
import shapely.wkb as wkblib

wkbfab = o.geom.WKBFactory()

class AmenityListHandler(o.SimpleHandler):

    def print_amenity(self, tags, lon, lat):
        name = tags.get('name', '')
        print("%f %f %-15s %s" % (lon, lat, tags['amenity'], name))

    def node(self, n):
        if 'amenity' in n.tags:
            self.print_amenity(n.tags, n.location.lon, n.location.lat)

    def area(self, a):
        if 'amenity' in a.tags:
            wkb = wkbfab.create_multipolygon(a)
            poly = wkblib.loads(wkb, hex=True)
            centroid = poly.representative_point()
            self.print_amenity(a.tags, centroid.x, centroid.y)


def main(osmfile):

    handler = AmenityListHandler()

    handler.apply_file(osmfile)

    return 0

if __name__ == '__main__':
    if len(sys.argv) != 2:
        print("Usage: python %s <osmfile>" % sys.argv[0])
        sys.exit(-1)

    exit(main(sys.argv[1]))
